module coarsen
! Coarsen mesh.
  use adj_mod
  use helixcartmod
!
!  constructs an xy array of the output point locations
!  Operator then calls the linear interpolation operator to
!  map the smaller gridded data to the expanded (in the sense
!  of area) ouput.
! 
  use lintn
  implicit none
  real,   dimension( :, :), pointer :: xyp 
  contains
  subroutine coarsen_init ( fo,fd,fn,co,cd,cn)

    integer                             :: in, ic, id, ie, dim, nd
    real,    dimension (:)              :: co, cd, fo, fd
    integer, dimension (:)              :: cn, fn
    real,    dimension (:), allocatable :: loc_fine
    real, dimension (:), allocatable :: nmin, nmax
    integer, dimension (:), allocatable :: iloc_fine

!                                                                           write(0,*)"coarsen_init 1"
    dim = size(fo)
    nd = product(fn)
    allocate (xyp(nd,dim),nmin(dim),nmax(dim))
    allocate (loc_fine(dim),iloc_fine(dim))
 !                                                                          write(0,*)"coarsen_init 2",size(xyp),"fn",fn
!
!                                    write(0,*)"fo",fo
!                                    write(0,*)"fd",fd
!                                    write(0,*)"fn",fn
    nmin = fo-fd/2.
    nmax = fo +fd*(fn-1.) +fd/2.        ! Great formula!
!                                    write(0,*)"nmax",nmax
!                                    write(0,*)"nmin",nmin
    cd = (nmax-nmin)/cn
    co = (nmin*(cn-.5) + nmax/2.)/cn    ! Great formula!
!                                    write(0,*)"co",co
!                                    write(0,*)"cd",cd
!                                    write(0,*)"cn",cn
!                                                                           write(0,*)"coarsen_init 3",fn,size(xyp)
!
                                                                            
   !loop through the fine space
   do ic=1,nd 
     call helix2cart(fn,ic,iloc_fine) ! my current index in fine space
     loc_fine= (iloc_fine-1.)*fd+fo ! my current position in fine space
     xyp(ic,:) = loc_fine
   end do 
 !                                                                          write(0,*)"coarsen_init 4"
    call lintn_init( cn, co, cd, xyp)
!                                                                           write(0,*)"coarsen_init 5"
  end subroutine
  function coarsen_lop ( adj, add, mm, dd) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: mm,dd 
!write(0,*)"coarsen_lop crap1"
    call adjnull (adj,add,mm,dd )
!write(0,*)"coarsen_lop crap2"
    call coarsen_lop2(adj,mm,dd )
    stat=0
  end function 
  subroutine coarsen_lop2(adj,mm,dd)
    logical,intent(in) :: adj
    real, dimension (:)  :: mm 
    real, dimension (:)  :: dd 
    integer stat1
!write(0,*)"coarsen_lop before mm",minval(mm),maxval(mm),size(mm)
!write(0,*)"coarsen_lop before dd",minval(dd),maxval(dd),size(dd)
    stat1 = lintn_lop( adj, .true., mm, dd)
!write(0,*)"coarsen_lop after mm",minval(mm),maxval(mm),size(mm)
!write(0,*)"coarsen_lop after dd",minval(dd),maxval(dd),size(dd)
  end subroutine
  subroutine  coarsen_close()
    deallocate (xyp)
    call lintn_close()
  end subroutine 
end module 














