!$
!$=head1 NAME
!$
!$helicon - convolution using helix filters
!$
!$=head1 SYNOPSIS
!$
!$Initializer - C<call helicon_init(aa)>
!$
!$Operator    - C<ierr=helicon_lop(adj,add,xx,yy)>
!$
!$=head1 PARAMETERS
!$
!$=over 4
!$
!$=item aa - type(filter)
!$
!$      helix filter to perform convolution with
!$
!$=item adj,add,xx,yy -
!$
!$      standard operators parameters
!$
!$=back
!$
!$=head1 DESCRIPTION
!$
!$ Convolution, inverse to deconvolution.
!$ Requires the filter be causal with an implicit "1." at the onset.
!$
!$
!$=head1 SEE ALSO
!$
!$L<helix>
!$
!$=head1 LIBRARY
!$
!$B<geef90>
!$
!$=cut
!$
module helicon_ir 
! Convolution, inverse to deconvolution.
  use adj_mod
!             Requires the filter be causal with an implicit "1." at the onset.
  use helix
  implicit none
  type( filter), private :: aa 
  real ,private,  dimension (:), pointer  :: d 
  integer, dimension (:), allocatable :: oldlag
  real, dimension (:,:), allocatable :: lagreal
  integer                          :: n1, n2, ixx
  real                               :: fr
  contains
  subroutine helicon_ir_init ( aa_in,d_in,n1_in )
    real, dimension (:), pointer :: d_in
    type( filter)    :: aa_in 
    integer          :: n1_in, i2, ia
    aa = aa_in
    d => d_in
    n1 = n1_in
    n2 = size(d)+1
    allocate(oldlag(size(aa%lag)),lagreal(n2,size(aa%lag)))
    oldlag = aa%lag
    do i2=1,n2
       do ia=1,size(oldlag)
          lagreal(i2,ia)=oldlag(ia)*d(i2)
          if (oldlag(ia)>n1/2) then
             lagreal(i2,ia) = lagreal(i2,ia)-n1*(d(i2)-1)
          end if
       end do
    end do
  end subroutine
  function helicon_ir_lop ( adj, add, xx, yy) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: xx,yy 
    call adjnull (adj,add,xx,yy )
    call helicon_ir_lop2(adj,xx,yy )
    stat=0
  end function 
  subroutine helicon_ir_lop2(adj,xx,yy)
    logical,intent(in) :: adj 
    real, dimension (:)  :: xx 
    real, dimension (:)  :: yy 
    integer iy, ix, ia
    if ( adj) then
! zero lag
      xx =	xx + yy
    else
      yy =	yy + xx
    end if 
do ix = 1,size(d)
   aa%lag=int(lagreal(ix,:))
    do ia = 1, size( aa%lag)
      do iy = 1  + aa%lag(ia), size( yy)
         if ( associated( aa%mis)) then
            if ( aa%mis( iy)) then
               cycle
            end if
         end if
         ixx = iy - aa%lag( ia)
         fr = lagreal(ix,ia)-aa%lag(ia)
         if ( adj) then
            !          xx(ix) = xx(ix) + (fr*yy(iy)+(1-fr)*yy(iy+1))*aa%flt(ia)
            xx(ixx) = xx(ixx) + fr*yy(iy)*aa%flt(ia)
            xx(ixx+1) = xx(ixx+1) + (1-fr)*yy(iy)*aa%flt(ia)
         else
            yy(iy) = yy(iy) + (fr*xx(ixx)+(1-fr)*xx(ixx+1))*aa%flt(ia)
         end if
      end do
   end do
end do
  end subroutine 
  subroutine helicon_ir_close()
  end subroutine 
end module 
